package btools.router.navi;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import rgnavigator.core.interfaces.NodeAttributeContainerI;
import btools.router.RoutingContext;

public class AttributeContainer implements NodeAttributeContainerI {
	private byte[] descriptionBitmap;
	private Map<String, String> attribMap;
	private boolean counterLinkWritten;
	private RoutingContext routingContext;

	public AttributeContainer(RoutingContext pRoutingContext, boolean pCounterLinkWritten, byte[] pDescriptionBitmap) {
		descriptionBitmap = pDescriptionBitmap;
		counterLinkWritten = pCounterLinkWritten;
		routingContext = pRoutingContext;
	}

	public Collection<String> getAttributes() {
		initAttributMap();
		return attribMap.keySet();
	}

	public String getAttribute(String pKey) {
		initAttributMap();
		return attribMap.get(pKey);
	}

	public boolean hasAttribute(String pKey) {
		initAttributMap();
		return attribMap.containsKey(pKey);
	}

	public boolean equals(NodeAttributeContainerI pNodeAttributeContainer) {
		boolean result = false;
		if (pNodeAttributeContainer instanceof AttributeContainer) {
			result = Arrays.equals(descriptionBitmap, ((AttributeContainer) pNodeAttributeContainer).descriptionBitmap);
		}
		return result;
	}

	private void initAttributMap() {
		if (attribMap == null) {
			attribMap = new HashMap<String, String>();
			if (descriptionBitmap != null) {
				String attribStrings = routingContext.expctxWay.getKeyValueDescription(counterLinkWritten, descriptionBitmap);
				StringTokenizer st = new StringTokenizer(attribStrings, " ");
				while (st.hasMoreTokens()) {
					String attribValuePair = st.nextToken();
					int index = attribValuePair.indexOf("=");
					if (index > 0) {
						String attrib = attribValuePair.substring(0, index);
						String value = attribValuePair.substring(index+1);
						attribMap.put(attrib, value);
					}
				}
			}
		}
	}

}
