package btools.router.navi;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import rgnavigator.core.factories.MapFactory;
import rgnavigator.core.factories.NavigatorFactory;
import rgnavigator.core.factories.WayFactory;
import rgnavigator.core.interfaces.NaviMapI;
import rgnavigator.core.interfaces.NaviNodeI;
import rgnavigator.core.interfaces.NaviWaypointI;
import rgnavigator.core.interfaces.TackNavigatorI;
import rgnavigator.core.interfaces.WayI;
import btools.mapaccess.OsmLink;
import btools.mapaccess.OsmNode;
import btools.mapaccess.OsmTransferNode;
import btools.router.OsmPathElement;
import btools.router.RoutingContext;

public class NaviBridge {

	// =========================================================================
	// ===========================================================================
	// ===========================================================================
	// ===========================================================================
	// ===========================================================================
	// ===========================================================================

	public List<NaviWaypointI> calculateNaviWaypoints(Set<NaviBridgeLink> nodes, ArrayList<OsmPathElement> pTrackNodes, RoutingContext pRoutingContext) {
		List<NaviWaypointI> result = null;
		if (pTrackNodes.size() > 1) {
			NaviMapI naviMap = MapFactory.getNaviMap();
			for (NaviBridgeLink node : nodes) {
				addWays(naviMap, node, pRoutingContext);
			}

			String baseDir = System.getProperty( "profileBaseDir" );
			
			TackNavigatorI tackNavigator = NavigatorFactory.createTackNavigator(naviMap, baseDir, pRoutingContext.localFunction);
			for (OsmPathElement trackNode : pTrackNodes) {
				NaviTrackElement naviTrackElement = new NaviTrackElement(trackNode);
				tackNavigator.addStep(naviTrackElement);
			}
			result =  tackNavigator.getNavigationResult();
		}

		return result;

	}

	// ======================================================================

	private void addWays(NaviMapI pNaviMap, NaviBridgeLink pNode, RoutingContext pRoutingContext) {

		ArrayList<NaviNodeI> wayData = getTrackElements(pNode.getSource(), pNode.getLink(), pRoutingContext);
		WayI way = WayFactory.createWay(wayData);
		
		pNaviMap.add(way);
	}


	// ======================================================================
	private ArrayList<NaviNodeI> getTrackElements(OsmNode pStart, OsmLink pLink, RoutingContext pRoutingContext) {
		ArrayList<NaviNodeI> elements = new ArrayList<NaviNodeI>();
		AttributeContainer attributeContainer = new AttributeContainer(pRoutingContext, pLink.counterLinkWritten, pLink.descriptionBitmap);
		AttributeContainer linkAttributeContainer = attributeContainer;
		NaviTrackElement naviTrackElement = new NaviTrackElement(pStart, attributeContainer);
		elements.add(naviTrackElement);
		AttributeContainer preAttributeContainer = attributeContainer;
		if (pLink.firsttransferBytes != null) {
			for (OsmTransferNode node = OsmTransferNode.decode(pLink.firsttransferBytes); node != null; node = node.next) {
				attributeContainer = new AttributeContainer(pRoutingContext, pLink.counterLinkWritten, node.descriptionBitmap);
				if(!preAttributeContainer.equals(attributeContainer)) {
					preAttributeContainer = attributeContainer;
				}
				naviTrackElement = new NaviTrackElement(node, preAttributeContainer);
				 
				elements.add(naviTrackElement);
			}
		}
		attributeContainer = linkAttributeContainer;
		if(!preAttributeContainer.equals(attributeContainer)) {
			preAttributeContainer = attributeContainer;
		}
		naviTrackElement = new NaviTrackElement(pLink.targetNode, preAttributeContainer);
		elements.add(naviTrackElement);
		return elements;
	}
}
