package rgnavigator.core;

import rgnavigator.core.interfaces.PosI;

public class NaviUtil {
	public static long getIdFromPos(int ilon, int ilat) {
		return ((long) ilon) << 32 | ilat;
	}

	// ===========================================================================
	public static long getIdFromPos(PosI pos) {
		return ((long) pos.getILon()) << 32 | pos.getILat();
	}

	// ===========================================================================

	public static int calcDistance(PosI p1, PosI p2) {
		double l = (p1.getILat() - 90000000) * 0.00000001234134;
		double l2 = l * l;
		double l4 = l2 * l2;
		double coslat = 1. - l2 + l4 / 6.;

		double dlat = (p1.getILat() - p2.getILat()) / 1000000.;
		double dlon = (p1.getILon() - p2.getILon()) / 1000000. * coslat;
		double d = Math.sqrt(dlat * dlat + dlon * dlon) * (6378000. / 57.);
		return (int) (d + 1.0);
	}

	// ===========================================================================
	//
	// public static void main(String[] args) {
	// NaviUtil naviUtil = new NaviUtil();
	// // Pos p1 = new Pos(7, 2, (short) 0);
	// // Pos p2 = new Pos(4, 2, (short) 0);
	// // Pos p3 = new Pos(2, 4, (short) 0);
	//
	// Pos p1 = new Pos(3,1, (short) 0);
	// Pos p2 = new Pos(4, 3, (short) 0);
	//
	// Pos p3 = new Pos(6, 2, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	// p3 = new Pos(4, 2, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	// p3 = new Pos(2, 2, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	//
	//
	// p1 = new Pos(3,5, (short) 0);
	// p2 = new Pos(4, 3, (short) 0);
	// p3 = new Pos(6, 2, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	// p3 = new Pos(4, 2, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	// p3 = new Pos(5, 4, (short) 0);
	// System.out.println(" Result = " + p3+" = " + naviUtil.calcAngleChange(p1,
	// p2, p3));
	//
	//
	//
	// double x = 1d;
	// double y = -5d;
	// System.out.println(" Result tst = " + Math.toDegrees(Math.atan(y / x)));
	//
	// }

	// ===========================================================================
	public static double calcAngleChange(PosI pos1, PosI pos2, PosI pos3) {
		double result = 0;

		double dlat1 = (pos2.getILat() - pos1.getILat());
		double dlon1 = (pos2.getILon() - pos1.getILon());
		double dlat2 = (pos3.getILat() - pos2.getILat());
		double dlon2 = (pos3.getILon() - pos2.getILon());

		double ang1 = calcAngle(dlon1, dlat1);
		double ang2 = calcAngle(dlon2, dlat2);

		if (ang1 == ang1 && ang2 == ang2) {
			result = ang2 - ang1;
			if (result > 180) {
				result = -360 + result;
			} else if (result < -180) {
				result = 360 + result;
			}
		}
		return result;
	}

	// ===========================================================================
	private static double calcAngle(double x, double y) {
		double ang = 0d;
		ang = Math.toDegrees(Math.atan(y / x));
		if (x < 0 && y >= 0) {
			ang = 180 + ang;
		} else if (x < 0 && y < 0) {
			ang = -180 + ang;
		}
		return ang;
	}
}
