package rgnavigator.core;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import rgnavigator.core.factories.WayFactory;
import rgnavigator.core.interfaces.NaviNodeI;
import rgnavigator.core.interfaces.PosI;
import rgnavigator.core.interfaces.WayI;
import rgnavigator.core.interfaces.WayIteratorI;
import rgnavigator.core.interfaces.WayNodeI;

public class Way extends Pos implements WayI {
	private long[] posArray;
	private List<WayNodeI> wayData;

	// private WayConnectorI start;
	// private WayConnectorI end;

	// ==========================================================

	public Way(List<NaviNodeI> pWayNodes) {
		wayData = new ArrayList<WayNodeI>();
		posArray = new long[pWayNodes.size()];

		int i;
		int diff;
		boolean orderCorrect = isCorrectPosOrder(pWayNodes);
		if (orderCorrect) {
			i = 0;
			diff = 1;
		} else {
			i = pWayNodes.size() - 1;
			diff = -1;
		}

		WayNodeI lastNode = null;
		for (NaviNodeI node : pWayNodes) {
			WayNodeI newNode = WayFactory.createWayNode(node, node.getNodeAttributeContainer(), this);
			if (orderCorrect) {
				wayData.add(newNode);
			} else {
				wayData.add(0, newNode);
			}
			posArray[i] = NaviUtil.getIdFromPos(newNode);

			if (lastNode != null) {
				newNode.dLinkNeighbours(lastNode);
			}

			i += diff;
			lastNode = newNode;
		}
		set(getStartWayNode());
	}

	// ==========================================================

	public WayNodeI getStartWayNode() {
		return !isValid() ? null : getNodes().get(0);
	}

	// ==========================================================

	public WayNodeI getEndWayNode() {
		return !isValid() ? null : getNodes().get(getNodes().size() - 1);
	}

	// ==========================================================
	@Override
	public List<WayNodeI> getNodes() {
		return wayData;
	}

	// ==========================================================
	@Override
	public WayIteratorI getWayIterator(PosI pPos) {
		boolean reverse;
		if (getNodes().get(0).equals(pPos)) {
			reverse = false;
		} else if (getNodes().get(getNodes().size() - 1).equals(pPos)) {
			reverse = true;
		} else {
			return null;
		}
		return new WayIterator(getNodes(), reverse);
	}

	// ==========================================================
	@Override
	public boolean isValid() {
		return getNodes() != null && getNodes().size() > 1;
	}

	// ==========================================================

	@Override
	public int hashCode() {
		return getILat() + getILon();
	}

	// ==========================================================
	@Override
	public boolean containsPos(PosI pPos) {
		boolean result = false;
		long checkId = NaviUtil.getIdFromPos(pPos);
		for (long id : posArray) {
			if (id == checkId) {
				result = true;
				break;
			}
		}
		return result;
	}

	// ==========================================================
	@Override
	public boolean equals(Object cmp) {
		boolean result = false;
		if (cmp instanceof WayI) {
			WayI way = (WayI) cmp;

			if (getNodes().size() == way.getNodes().size()) {
				if (way instanceof Way) {
					result = Arrays.equals(posArray, ((Way) way).posArray);
				} else {
					result = true;
					for (int i = 0; i < getNodes().size(); i++) {
						if (NaviUtil.getIdFromPos(getNodes().get(i)) != NaviUtil.getIdFromPos(way.getNodes().get(i))) {
							result = false;
							break;
						}
					}
				}
			}
		} else {
			super.equals(cmp);
		}
		return result;
	}

	// ==========================================================
	// ==========================================================
	// ==========================================================
	/**
	 * Check, if the way starts with the smaller positions. If so, the order is
	 * correct. If not, we need to reverse the order
	 *
	 * @param pWayNodes
	 * @return
	 */

	private boolean isCorrectPosOrder(List<NaviNodeI> pWayNodes) {
		int compResult = 0;
		int size = pWayNodes.size();
		for (int i = 0; i < pWayNodes.size() / 2; i++) {
			compResult = pWayNodes.get(i).compare(pWayNodes.get(size - 1 - i));
			if (compResult != 0) {
				break;
			}
		}
		return compResult <= 0;
	}

}
