package rgnavigator.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import rgnavigator.core.interfaces.NodeAttributeContainerI;
import rgnavigator.core.interfaces.PosI;
import rgnavigator.core.interfaces.WayI;
import rgnavigator.core.interfaces.WayNodeI;

public class WayNode extends Pos implements WayNodeI {
	private WayI way;
	private List<WayNodeI> neighbours;
	private NodeAttributeContainerI nodeAttributeContainer;
	
	// ==========================================================

	public WayNode(PosI pPos, NodeAttributeContainerI pNodeAttributeContainer, WayI pWay) {
		super(pPos);
		nodeAttributeContainer = pNodeAttributeContainer;
		// +++
		// neighbours = new CopyOnWriteArrayList<WayNodeI>();
		neighbours = new ArrayList<WayNodeI>();
		way = pWay;
	}

	// ==========================================================
	@Override
	public Collection<WayNodeI> getNeighbours() {
		return neighbours;
	}

	// ==========================================================

	@Override
	public void addNeihgbour(WayNodeI pNode) {
		neighbours.add(pNode);
	}

	// ==========================================================
	@Override
	public void dLinkNeighbours(WayNodeI pNode) {
//		if(!pNode.getWay().containsPos(this)) {
//			System.out.println("Sehr komisch !");
//		}
		addNeihgbour(pNode);
		pNode.addNeihgbour(this);
	}

	// ==========================================================
	@Override
	public void dLinkAllNeighbours(Collection<WayNodeI> pNodes) {
		for (WayNodeI node : pNodes) {
			dLinkNeighbours(node);
		}
	}

	// ==========================================================

	/**
	 * Get the next node on the same way for the given position
	 */
	public WayNodeI getNextWayNodeForPos(PosI pPos) {
		WayNodeI result = null;
		for (WayNodeI checkNode : getNeighbours()) {
			if (checkNode.equals(pPos) && checkNode.getWay() == getWay()) {
				result = checkNode;
				break;
			}
		}

		return result;

	}

	// ==========================================================

	@Override
	public boolean containsNeihgbour(WayNodeI pNode) {
		boolean result = false;
		for (WayNodeI checkNode : getNeighbours()) {
			if (checkNode.equals(pNode)) {
				result = true;
				break;
			}
		}
		return result;
	}

	
	// ==========================================================

	@Override
	public WayI getWay() {
		return way;
	}

	// ==========================================================

	// @Override
	// public void setWay(WayI pWay) {
	// way = pWay;
	// }

	// ==========================================================

	@Override
	public WayNodeI clone() {
		return (WayNodeI) super.clone();
	}

	// ==========================================================

	public boolean equals(Object cmp) {
		boolean result = false;

		if (cmp instanceof WayNodeI) {
			result = super.equals((WayNodeI) cmp) && getWay().equals(((WayNodeI) cmp).getWay());
		} 
//		else {
//			result = super.equals(cmp);
//		}
		return result;
	}

	
	// ==========================================================
	
	@Override
	public NodeAttributeContainerI getNodeAttributeContainer() {
		return nodeAttributeContainer;

	}
	
	
	
}
