package rgnavigator.map;

import java.util.Collection;
import java.util.HashSet;
import java.util.Queue;
import java.util.Set;

import rgnavigator.core.exceptions.NotImplementedException;
import rgnavigator.core.factories.WayFactory;
import rgnavigator.core.interfaces.MapObjectI;
import rgnavigator.core.interfaces.MapTileI;
import rgnavigator.core.interfaces.PosI;
import rgnavigator.core.interfaces.WayConnectorI;
import rgnavigator.core.interfaces.WayI;
import rgnavigator.core.interfaces.WayNodeI;

public class MapTile implements MapTileI {
	NodePosMap<MapObjectI> map = new NodePosMap<MapObjectI>();
	Set<WayI> waySet = new HashSet<WayI>();

	// ==========================================================
	@Override
	public Collection<MapObjectI> getMapObjects(PosI pPos) {
		return map.getNodesForPos(pPos);
	}

	// ==========================================================

	public Collection<MapObjectI> getAllMapObjects() {
		return map.getNodes();
	}

	// ==========================================================

	@Override
	public void add(MapObjectI pMapObject) {

		if (pMapObject instanceof WayI) {
			addWay((WayI) pMapObject);

		} else {

			throw new NotImplementedException();
		}
	}

	// ==========================================================

	@Override
	public void add(Collection<MapObjectI> pMapObject) {
		throw new NotImplementedException();
	}

	// ==========================================================
	@Override
	public WayConnectorI getFirstWayConnector(PosI pPos) {
		WayConnectorI result = null;
		Collection<MapObjectI> list = getMapObjects(pPos);
		if (list != null) {
			for (MapObjectI mapObject : list) {
				if (mapObject instanceof WayConnectorI) {
					result = (WayConnectorI) mapObject;
					break;
				}
			}
		}
		return result;
	}

	// ==========================================================
	@Override
	public WayNodeI getFirstWayNode(PosI pPos) {
		WayNodeI result = null;
		Collection<MapObjectI> list = getMapObjects(pPos);
		if (list != null) {
			for (MapObjectI mapObject : list) {
				if (mapObject instanceof WayNodeI) {
					result = (WayNodeI) mapObject;
					break;
				}
			}
		}
		return result;
	}

	// ==========================================================
	private void addWay(WayI pWay) {
		if (pWay.isValid()) {
			// WayNodeI startPos = pWay.getStartWayNode();
			// WayNodeI endPos = pWay.getEndWayNode();

			synchronized (map) {
				if (!waySet.contains(pWay)) {
					waySet.add(pWay);
					for (WayNodeI node : pWay.getNodes()) {
						map.putNode(node);
					}
					addConnector(pWay.getStartWayNode());
					addConnector(pWay.getEndWayNode());
				}
			}
		}
	}

	// ==========================================================

	private void addConnector(WayNodeI pNode) {
		WayConnectorI wayConnector = getFirstWayConnector(pNode);
		if (wayConnector == null) {
			wayConnector = WayFactory.createWayConnector(pNode);
			map.putNode(wayConnector);
		}
		wayConnector.addWayNode(pNode);
	}

	// ==========================================================

	/**
	 * Check, if we already have a connector. If not, create a new one and add
	 * it to the map. If we found one, we add the new node and connect the node
	 * to the other nodes
	 * 
	 * @param pNode
	 */
	private void handleWayConnector(WayNodeI pNode) {
		boolean newlyCreated = false;

		WayConnectorI connector = getWayConnection(pNode);
		if (connector == null) {
			connector = WayFactory.createWayConnector(pNode);
			newlyCreated = true;
		}
		if (!connector.contains(pNode)) {
			for (WayNodeI otherNode : connector.getWayNodes()) {
				pNode.dLinkNeighbours(otherNode);
			}
			connector.addWayNode(pNode);
		}
		if (newlyCreated) {
			map.putNode(connector);
		}
	}

	// ==========================================================

	private WayConnectorI getWayConnection(PosI pPos) {
		WayConnectorI result = null;

		Queue<MapObjectI> queue = map.getNodesForPos(pPos);
		if (queue != null) {
			for (MapObjectI obj : queue) {
				if (obj instanceof WayConnectorI) {
					result = (WayConnectorI) obj;
					break;
				}
			}
		}
		return result;
	}
}
