package rgnavigator.map;

import java.util.ArrayList;
import java.util.Queue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import rgnavigator.core.NaviUtil;
import rgnavigator.core.interfaces.MapObjectI;
import rgnavigator.core.interfaces.PosI;

class NodePosMap<T extends MapObjectI> {
	private ConcurrentHashMap<Long, ConcurrentLinkedQueue<T>> posMap;

	// ==============================================================
	public NodePosMap() {
		posMap = new ConcurrentHashMap<Long, ConcurrentLinkedQueue<T>>();
	}

	// ==============================================================
	/**
	 * Only add nodes if, it is not already added
	 * 
	 * 
	 * @param pNode
	 */
	public void putNode(T pNode) {
		long id = NaviUtil.getIdFromPos(pNode);
		synchronized (posMap) {
			Queue<T> current = posMap.get(id);
			if (current != null) {
				boolean found = false;
				for (T node : getNodesForPos(id)) {
					if (node.equals(pNode)) {
						found = true;
						break;
					}
				}
				if (!found) {
					current.add(pNode);
				}
			} else {
				ConcurrentLinkedQueue<T> newQueue = new ConcurrentLinkedQueue<T>();
				newQueue.add(pNode);
				posMap.put(id, newQueue);
			}
		}
	}

	// ==============================================================
	public boolean containsPos(PosI pPos) {
		return posMap.containsKey(NaviUtil.getIdFromPos(pPos));
	}

	// ==============================================================
	/**
	 * Check all objects on this position with == . So only the existence of this instance is checked
	 * @param pNode
	 * @return
	 */
	
	public boolean containsInstance(T pNode) {
		Queue<T> queue = posMap.get(NaviUtil.getIdFromPos(pNode));
		return queue != null && queue.contains(pNode);
	}

	// ==============================================================
	/**
	 * Check all objects on this position with equals()
	 * @param pNode
	 * @return
	 */
	
	public boolean containsContent(T pNode) {
		boolean result = false;
		Queue<T> queue = posMap.get(NaviUtil.getIdFromPos(pNode));
		if (queue != null) {
			for (T obj : queue) {
				if(obj.equals(pNode)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

	// ==============================================================
	public T getFirstNodeForPos(PosI pPos) {
		T result = null;

		Queue<T> queue = posMap.get(NaviUtil.getIdFromPos(pPos));
		if (queue != null) {
			result = queue.peek();
		}
		return result;
	}

	// ==============================================================
	/**
	 * Get the no of entries at this pos
	 * 
	 * @param pPos
	 * @return
	 */
	public int size(long pPosId) {
		int result = 0;
		Queue<T> queue = posMap.get(pPosId);
		if (queue != null) {
			result = queue.size();
		}
		return result;
	}

	// ==============================================================
	/**
	 * Get the no of entries at this pos
	 * 
	 * @param pPos
	 * @return
	 */
	public int size(PosI pPos) {
		return size(NaviUtil.getIdFromPos(pPos));
	}

	// ==============================================================
	public boolean isEmpty() {
		return posMap.isEmpty();
	}

	// ==============================================================
	public boolean containsPos(long pPosId) {
		return posMap.containsKey(pPosId);
	}

	// ==============================================================
	public Queue<T> getNodesForPos(long pPosId) {
		return posMap.get(pPosId);
	}

	// ==============================================================
	public Queue<T> getNodesForPos(PosI pPos) {
		return posMap.get(NaviUtil.getIdFromPos(pPos));
	}

	// ==============================================================
	public void clear() {
		posMap.clear();
	}

	// ==============================================================
	public ArrayList<T> getNodes() {
		ArrayList<T> result = new ArrayList<T>();

		for (ConcurrentLinkedQueue<T> queue : posMap.values()) {
			result.addAll(queue);
		}
		return result;
	}

}
