package rgnavigator.navigation;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import rgnavigator.core.Pos;
import rgnavigator.core.exceptions.InvalidAttributeException;
import rgnavigator.core.interfaces.NaviWaypointI;
import rgnavigator.core.interfaces.PosI;

public class NaviWaypoint extends Pos implements NaviWaypointI {
	private String name;
	private String comment;
	private String desc;
	private String symbol;
	private String type;
	private String gpx_extension;
	private String reference;

	public NaviWaypoint() {

	}

	public NaviWaypoint(PosI pPos) {
		super(pPos);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getName()
	 */
	@Override
	public String getName() {
		return name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setName(java.lang.String)
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getComment()
	 */
	@Override
	public String getComment() {
		return comment;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setComment(java.lang.String)
	 */
	@Override
	public void setComment(String comment) {
		this.comment = comment;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getDesc()
	 */
	@Override
	public String getDesc() {
		return desc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setDesc(java.lang.String)
	 */
	@Override
	public void setDesc(String desc) {
		this.desc = desc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getSymbol()
	 */
	@Override
	public String getSymbol() {
		return symbol;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setSymbol(java.lang.String)
	 */
	@Override
	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getType()
	 */
	@Override
	public String getType() {
		return type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setType(java.lang.String)
	 */
	@Override
	public void setType(String type) {
		this.type = type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#getGpxExtension()
	 */
	@Override
	public String getGpxExtension() {
		return gpx_extension;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * rgnavigator.navigation.NaviWaypointI#setGpxExtension(java.lang.String)
	 */
	@Override
	public void setGpxExtension(String extension) {
		this.gpx_extension = extension;
	}

	public String getReference() {
		return reference;
	}

	public void setReference(String reference) {
		this.reference = reference;
	}

	// ==================================================================
	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NaviWaypointI#setValue(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void setValue(String pAttribName, String pValue) {
		try {
			String methodName = getMethodFromAttrib(pAttribName, false);
			Method method = NaviWaypoint.class.getMethod(methodName, pValue.getClass());
			method.invoke(this, pValue);
		} catch (InvocationTargetException | SecurityException | IllegalArgumentException | IllegalAccessException | NoSuchMethodException e) {
			throw new InvalidAttributeException("Invalid attribute " + pAttribName, e);
		}
	}

	// ===============================================================

	@Override
	public String getValue(String pAttribName) {
		String result = null;
		try {
			String methodName = getMethodFromAttrib(pAttribName, true);
			Method method = NaviWaypoint.class.getMethod(methodName);
			result = (String) method.invoke(this);
		} catch (InvocationTargetException | SecurityException | IllegalArgumentException | IllegalAccessException | NoSuchMethodException e) {
			throw new InvalidAttributeException("Invalid attribute: " + pAttribName, e);
		}
		return result;
	}

	// ===============================================================

	private String getMethodFromAttrib(String pAttrib, boolean pGetter) {
		String firstChar = pAttrib.substring(0, 1);
		firstChar = firstChar.toUpperCase();
		String methodName = pGetter ? "get" : "set" + firstChar + pAttrib.substring(1);
		return methodName;
	}

}
