package rgnavigator.navigation;

import java.util.ArrayList;
import java.util.List;

import rgnavigator.core.interfaces.NaviWaypointI;
import rgnavigator.core.interfaces.NavigationContextI;
import rgnavigator.core.interfaces.TrackStepI;

public class NavigationContext implements NavigationContextI {
	private List<TrackStepI> steps = new ArrayList<TrackStepI>(200);
	private List<NaviWaypointI> waypoints = new ArrayList<NaviWaypointI>();
	private int sourceStepIndex = 0;

	// ===================================================================

	public NavigationContext() {
	}

	// ===================================================================

	@Override
	public List<TrackStepI> getSteps() {
		return steps;
	}

	@Override
	public boolean  hasSteps() {
		return steps != null && steps.size()>0;
	}

	@Override
	public List<NaviWaypointI> getWaypoints() {
		return waypoints;
	}

	@Override
	public void addWaypoint(NaviWaypointI pNaviWaypoint) {
		waypoints.add(pNaviWaypoint);
	}

	public int getSourceStepIndex() {
		return sourceStepIndex;
	}

	public void incSourceStepIndex() {
		this.sourceStepIndex++;
	}

	// ===================================================================
	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NavigationContextI#hasPrevPos()
	 */
	@Override
	public boolean hasPrevPos() {
		return getSteps() != null && !getSteps().isEmpty();
	}

	// ===================================================================
	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NavigationContextI#size()
	 */
	@Override
	public int stepCount() {
		return getSteps().size();
	}

	// ===================================================================
	/*
	 * (non-Javadoc)
	 * 
	 * @see rgnavigator.navigation.NavigationContextI#getPrevStep(int)
	 */
	@Override
	public TrackStepI getPrevStep(int pLookBackValue) {
		TrackStepI result = null;
		if (pLookBackValue <= getSteps().size() - 1) {
			result = getSteps().get(getSteps().size() - 1 - pLookBackValue);
		}
		return result;
	}

	// ===================================================================
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * rgnavigator.navigation.NavigationContextI#addStep(rgnavigator.core.interfaces
	 * .TrackStepI)
	 */
	@Override
	public void addStep(TrackStepI pStep) {
		if (getSteps().size() > 0) {
			TrackStepI last = getSteps().get(getSteps().size() - 1);
			pStep.setPrevious(last);
			last.setNext(pStep);
		}
		pStep.setStepNumber(getSteps().size());
		pStep.setSourceStepIndex(getSourceStepIndex());
		getSteps().add(pStep);
	}

	// ===================================================================

}
