package rgnavigator.navigation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import rgnavigator.core.Pos;
import rgnavigator.core.interfaces.PosI;
import rgnavigator.core.interfaces.TrackStepI;
import rgnavigator.core.interfaces.WayNodeI;

public class TrackStep extends Pos implements TrackStepI {
	private WayNodeI wayNode;
	private int distance;
	private int stepLength;
	private double angleChange;
	private List<TrackStepI> stepAlternatives;
	private TrackStepI next;
	private TrackStepI previous;
	private int stepNo;
	private int sourceStepNo;

	// ====================================================================
	public TrackStep(PosI pPos, WayNodeI pWayNode) {
		super(pPos);
		wayNode = pWayNode;
	}

	// ====================================================================
	@Override
	public WayNodeI getWayNode() {
		return wayNode;
	}

	// ====================================================================
	@Override
	public int getDistance() {
		return distance;
	}

	// ====================================================================
	@Override
	public int getStepLength() {
		return stepLength;
	}

	// ====================================================================
	@Override
	public double getAngleChange() {
		return angleChange;
	}

	// ====================================================================
	public void setDistance(int distance) {
		this.distance = distance;
	}

	// ====================================================================
	public void setStepLength(int stepLength) {
		this.stepLength = stepLength;
	}

	// ====================================================================

	public void setAngleChange(double angleChange) {
		this.angleChange = angleChange;
	}

	// ====================================================================

	public Collection<WayNodeI> getNeighbours() {
		Collection<WayNodeI> result;
		if (getWayNode() != null) {
			result = getWayNode().getNeighbours();
		} else {
			result = new ArrayList<WayNodeI>();
		}
		return result;
	}

	// ===================================================================
	@Override
	public List<TrackStepI> getStepAlternatives() {
		return stepAlternatives;
	}

	// ===================================================================
	@Override
	public void addAlternative(TrackStepI pAlternativeStep) {
		if(stepAlternatives == null) {
			stepAlternatives = new ArrayList<TrackStepI>();
		}
		stepAlternatives.add(pAlternativeStep);
	}

	// ===================================================================
	@Override
	public void setStepAlternatives(List<TrackStepI> stepAlternatives) {
		this.stepAlternatives = stepAlternatives;
	}

	// ===================================================================
	@Override
	public boolean hasStepAlternatives() {
		return stepAlternatives != null && stepAlternatives.size() > 0;
	}

	@Override
	public TrackStepI getNext() {
		return next;
	}

	@Override
	public TrackStepI getPrevious() {
		return previous;
	}

	@Override
	public boolean hasNext() {
		return next != null;
	}

	@Override
	public boolean hasPrevious() {
		return previous != null;
	}
	
	@Override
	public int getStepNumber() {
		return stepNo;
	}

	@Override
	public void setStepNumber(int pNo) {
		stepNo = pNo;
	}

	@Override
	public void setNext(TrackStepI next) {
		this.next = next;
	}

	@Override
	public void setPrevious(TrackStepI previous) {
		this.previous = previous;
	}

	public int getSourceStepIndex() {
		return sourceStepNo;
	}

	public void setSourceStepIndex(int sourceStepNo) {
		this.sourceStepNo = sourceStepNo;
	}
	
}
