package rgnavigator.scripting;

import java.util.List;

import rgnavigator.core.NaviUtil;
import rgnavigator.core.factories.NaviWaypointFactory;
import rgnavigator.core.interfaces.NaviWaypointI;
import rgnavigator.core.interfaces.NodeAttributeContainerI;
import rgnavigator.core.interfaces.TrackStepI;
import sisc.data.Pair;
import sisc.data.Quantity;
import sisc.data.SchemeBoolean;
import sisc.data.SchemeString;
import sisc.data.Value;
import sisc.interpreter.ContinuationException;
import sisc.modules.s2j.Util;
import sisc.nativefun.IndexedFixableProcedure;
import sisc.nativefun.IndexedLibraryAdapter;

public class NaviProcedures extends IndexedFixableProcedure implements NaviProcedureIds {
	// ==================================================================
	public static class Index extends IndexedLibraryAdapter {
		public Value construct(Object o, int id) {
			return new NaviProcedures(id);
		}

		public Index() {
			define("step?", STEP_IS_STEP);
			define("step-next", STEP_NEXT_STEP);
			define("step-prev", STEP_PREV_STEP);
			define("step-pos", STEP_POS);
			define("pos?", STEP_IS_POS);
			define("step-alts", STEP_ALTERNATIVES);
			define("step-no", STEP_NO);
			define("step-source-step-no", STEP_SOURCE_STEP_NO);
			define("step-angle", STEP_ANGLE);
			define("step-length", STEP_LENGTH);
			define("step-distance", STEP_DISTANCE);
			define("step-elev", STEP_ELE);
			define("step-junction?", STEP_JUNCTION);
			define("step-next-junction", STEP_NEXT_JUNCTION);
			define("step-prev-junction", STEP_PREV_JUNCTION);
			define("step-count", STEP_COUNT);
			define("step-attrib-container", STEP_ATTRIBUTE_CONT);
			

			define("string-regex-replace-all", STRING_REPLACE_ALL);
			define("string-regex-replace-first", STRING_REPLACE_FIRST);

			define("waypoint-new", WAYPOINT_NEW);
			define("waypoint-value", WAYPOINT_GET_VALUE);
			define("waypoint-value!", WAYPOINT_SET_VALUE);
			define("waypoint-add", WAYPOINT_ADD);

			define("firststep", NAVICONTEXT_FIRST_STEP);
			define("laststep", NAVICONTEXT_LAST_STEP);
			define("navicontext?", NAVICONTEXT_IS_CONTEXT);
			define("hassteps", NAVICONTEXT_HAS_STEPS);

			define("attrib-container?", ATTRIBCONT_IS_ATTRIBUTE_CONT);
			define("attrib-container-all-attribs", ATTRIBCONT_ATTRIBUTES);
			define("attrib-container-attrib?", ATTRIBCONT_HAS_ATTRIBUTE);
			define("attrib-container-attrib", ATTRIBCONT_GET_ATTRIBUTE);
		}
	}

	// ==================================================================
	// ==================================================================
	// ==================================================================

	public NaviProcedures() {
	}

	// ==================================================================

	public NaviProcedures(int id) {
		super(id);
	}

	// ==================================================================
	public final Value apply() throws ContinuationException {
		Value result = VOID;

		switch (id) {
		default:
			throwArgSizeException();
		}
		return result;
	}

	// ==================================================================
	@Override
	public final Value apply(Value pValue) throws ContinuationException {
		Value result = VOID;

		switch (id) {
		case NAVICONTEXT_FIRST_STEP:
			result = firstStep(pValue);
			break;
		case NAVICONTEXT_LAST_STEP:
			result = lastStep(pValue);
			break;
		case STEP_NEXT_STEP:
			result = nextStep(pValue);
			break;
		case STEP_PREV_STEP:
			result = prevStep(pValue);
			break;
		case STEP_POS:
			result = newPos(pValue);
			break;
		case STEP_ALTERNATIVES:
			result = alts(pValue);
			break;
		case STEP_IS_POS:
			result = FALSE;
			if (pValue instanceof PosValue)
				result = TRUE;
			break;
		case STEP_IS_STEP:
			result = FALSE;
			if (pValue instanceof TrackStepValue)
				result = TRUE;
			break;
		case STEP_NO:
			result = stepNo(pValue);
			break;
		case STEP_SOURCE_STEP_NO:
			result = stepSourceStepNo(pValue);
			break;
		case STEP_ANGLE:
			result = angle(pValue);
			break;
		case STEP_LENGTH:
			result = length(pValue);
			break;
		case STEP_DISTANCE:
			result = dist(pValue);
			break;
		case STEP_ELE:
			result = elev(pValue);
			break;
		case STEP_JUNCTION:
			result = FALSE;
			if (pValue instanceof TrackStepValue) {
				if (((TrackStepValue) pValue).getTrackStep().hasStepAlternatives())
					result = TRUE;
			}
			break;
		case STEP_NEXT_JUNCTION:
			result = nextJunction(pValue);
			break;
		case STEP_PREV_JUNCTION:
			result = prevJunction(pValue);
			break;
		case WAYPOINT_NEW:
			result = newNaviWaypoint(pValue);
			break;
		case STEP_COUNT:
			result = getStepCount(pValue);
			break;
		case STEP_ATTRIBUTE_CONT:
			result = getAttribContainer(pValue);
			break;
		case ATTRIBCONT_IS_ATTRIBUTE_CONT:
			result = FALSE;
			if (pValue instanceof NodeAttributeContainerValue) {
					result = TRUE;
			}
			break;
		case ATTRIBCONT_ATTRIBUTES:
			result = getAllAttribsFromContainer(pValue);
			break;

		case NAVICONTEXT_IS_CONTEXT:
			result = FALSE;
			if (pValue instanceof NavigationContextValue)
				result = TRUE;
			break;
			
		case NAVICONTEXT_HAS_STEPS:
			result = FALSE;
			NavigationContextValue context = navigationContext(pValue);
			int size = context.getNavigationContext().stepCount();
			if(size > 0) {
				result = TRUE;
			}
			break;

		default:
			throwArgSizeException();
		}
		return result;
	}

	// ==================================================================

	@Override
	public final Value apply(Value pValue1, Value pValue2) throws ContinuationException {
		Value result = VOID;

		switch (id) {
		case STEP_AIR_DIST:
			result = airDistance(pValue1, pValue2);
			break;
		case WAYPOINT_GET_VALUE:
			result = getWaypointValue(pValue1, pValue2);
			break;
		case WAYPOINT_ADD:
			result = addWaypoint(pValue1, pValue2);
			break;
		case ATTRIBCONT_HAS_ATTRIBUTE:
			result = hasAttribute(pValue1, pValue2);
			break;
		case ATTRIBCONT_GET_ATTRIBUTE:
			result = getAttribute(pValue1, pValue2);
			break;
		default:
			throwArgSizeException();
		}
		return result;
	}

	// ==================================================================

	@Override
	public final Value apply(Value pValue1, Value pValue2, Value pValue3) throws ContinuationException {
		Value result = VOID;

		switch (id) {
		case STRING_REPLACE_ALL:
			result = stringReplaceAll(pValue1, pValue2, pValue3);
			break;
		case STRING_REPLACE_FIRST:
			result = stringReplaceFirst(pValue1, pValue2, pValue3);
			break;
		case WAYPOINT_SET_VALUE:
			result = setWaypointValue(pValue1, pValue2, pValue3);
			break;

		default:
			throwArgSizeException();
		}
		return result;
	}

	// ==================================================================
	// ==================================================================

	private NaviWaypointValue naviWaypoint(Value pValue) {
		if (pValue instanceof NaviWaypointValue) {
			return (NaviWaypointValue) pValue;
		} else {
			Util.typeError("naviWaypoint", pValue);
			return null;
		}
	}

	// ==================================================================

	private TrackStepValue trackStep(Value pValue) {
		if (pValue instanceof TrackStepValue) {
			return (TrackStepValue) pValue;
		} else {
			Util.typeError("trackStep", pValue);
			return null;
		}
	}

	// ==================================================================

	private PosValue pos(Value pValue) {
		if (pValue instanceof PosValue) {
			return (PosValue) pValue;
		} else {
			Util.typeError("pos", pValue);
			return null;
		}
	}

	// ==================================================================

	private NavigationContextValue navigationContext(Value pValue) {
		if (pValue instanceof NavigationContextValue) {
			return (NavigationContextValue) pValue;
		} else {
			Util.typeError("NavigationContext", pValue);
			return null;
		}
	}


	// ==================================================================

	private NodeAttributeContainerValue nodeAttributeContainer(Value pValue) {
		if (pValue instanceof NodeAttributeContainerValue) {
			return (NodeAttributeContainerValue) pValue;
		} else {
			Util.typeError("NodeAttributeContainer", pValue);
			return null;
		}
	}

	// ==================================================================
	// === NavigationContext=============================================
	// ==================================================================

	private Value getStepCount(Value pValue) {
		NavigationContextValue context = navigationContext(pValue);
		int size = context.getNavigationContext().stepCount();
		return Quantity.valueOf(size);
	}

	// ==================================================================

	private Value addWaypoint(Value pValue1, Value pValue2) {
		NavigationContextValue context = navigationContext(pValue1);
		NaviWaypointValue waypoint = naviWaypoint(pValue2);
		context.getNavigationContext().addWaypoint(waypoint.getWaypoint());
		return VOID;
	}

	// ==================================================================

	private Value firstStep(Value pValue) {
		Value result = VOID;
		NavigationContextValue context = navigationContext(pValue);
		List<TrackStepI> steps = context.getNavigationContext().getSteps();
		if (steps.size() > 0) {
			result = new TrackStepValue(steps.get(0));
		}
		return result;
	}

	// ==================================================================

	private Value lastStep(Value pValue) {
		Value result = VOID;
		NavigationContextValue context = navigationContext(pValue);
		List<TrackStepI> steps = context.getNavigationContext().getSteps();
		if (steps.size() > 0) {
			result = new TrackStepValue(steps.get(steps.size() - 1));
		}
		return result;
	}

	// ==================================================================
	// === String =======================================================
	// ==================================================================

	private Value stringReplaceAll(Value pValue1, Value pValue2, Value pValue3) {
		SchemeString result = new SchemeString();
		String s1 = Util.str(pValue1).asString();
		String s2 = Util.str(pValue2).asString();
		String s3 = Util.str(pValue3).asString();
		String resultStr = s1.replaceAll(s2, s3);
		result.set(resultStr);
		return result;
	}

	// ==================================================================

	private Value stringReplaceFirst(Value pValue1, Value pValue2, Value pValue3) {
		SchemeString result = new SchemeString();
		String s1 = Util.str(pValue1).asString();
		String s2 = Util.str(pValue2).asString();
		String s3 = Util.str(pValue3).asString();
		String resultStr = s1.replaceFirst(s2, s3);
		result.set(resultStr);
		return result;
	}

	// ==================================================================
	// == NaviWaypoint ==================================================
	// ==================================================================

	private Value setWaypointValue(Value pValue1, Value pValue2, Value pValue3) {
		NaviWaypointValue naviWaypointValue = naviWaypoint(pValue1);
		String attribName = Util.symval(pValue2);
		SchemeString value = Util.str(pValue3);
		naviWaypointValue.setValue(attribName, value.asString());

		return naviWaypointValue;
	}

	// ==================================================================

	private Value getWaypointValue(Value pValue1, Value pValue2) {
		NaviWaypointValue naviWaypointValue = naviWaypoint(pValue1);
		String attribName = Util.symval(pValue2);
		String value = naviWaypointValue.getValue(attribName);
		return new SchemeString(value);
	}

	// ==================================================================
	// == Step ==========================================================
	// ==================================================================

	
	private Value getAttribContainer(Value pValue) {
		Value result = VOID;

		TrackStepValue tracksep = trackStep(pValue);
		NodeAttributeContainerI nodeAttributeContainer = tracksep.getTrackStep().getWayNode().getNodeAttributeContainer();
		result = new NodeAttributeContainerValue(nodeAttributeContainer);
		return result;
	}
	
	
	// ==================================================================

	private Value airDistance(Value pValue1, Value pValue2) {
		Value result = VOID;

		PosValue posval1 = (PosValue) newPos(pValue1);
		PosValue posval2 = (PosValue) newPos(pValue2);

		result = Quantity.valueOf(NaviUtil.calcDistance(posval1.getPos(), posval2.getPos()));
		return result;
	}

	// ==================================================================

	private Value newNaviWaypoint(Value pValue) {
		Value result = VOID;

		PosValue posValue = pos(pValue);
		NaviWaypointI waypoint = NaviWaypointFactory.createNaviWaypoint(posValue.getPos());
		result = new NaviWaypointValue(waypoint);
		return result;
	}

	// ==================================================================

	private Value nextStep(Value pValue) {
		Value result = EMPTYLIST;
		TrackStepValue trackStepValue = trackStep(pValue);
		TrackStepI next = trackStepValue.getTrackStep().getNext();
		if (next != null) {
			result = new TrackStepValue(next);
		}
		return result;
	}

	// ==================================================================

	private Value prevStep(Value pValue) {
		Value result = EMPTYLIST;
		TrackStepValue trackStepValue = trackStep(pValue);
		TrackStepI prev = trackStepValue.getTrackStep().getPrevious();
		if (prev != null) {
			result = new TrackStepValue(prev);
		}
		return result;
	}

	// ==================================================================

	private Value nextJunction(Value pValue) {
		Value result = EMPTYLIST;
		TrackStepValue trackStepValue = trackStep(pValue);
		for (TrackStepI step = trackStepValue.getTrackStep().getNext(); step != null; step = step.getNext()) {
			if (step.hasStepAlternatives()) {
				result = new TrackStepValue(step);
				break;
			}
		}
		return result;
	}

	// ==================================================================

	private Value prevJunction(Value pValue) {
		Value result = EMPTYLIST;
		TrackStepValue trackStepValue = trackStep(pValue);
		for (TrackStepI step = trackStepValue.getTrackStep().getPrevious(); step != null; step = step.getPrevious()) {
			if (step.hasStepAlternatives()) {
				result = new TrackStepValue(step);
				break;
			}
		}
		return result;
	}

	// ==================================================================

	private Value newPos(Value pValue) {
		Value result = VOID;
		if (pValue instanceof PosValue) {
			result = new PosValue(((PosValue) pValue).getPos());
		} else if (pValue instanceof TrackStepValue) {
			result = new PosValue(((TrackStepValue) pValue).getTrackStep());
		} else if (pValue instanceof NaviWaypointValue) {
			result = new PosValue(((NaviWaypointValue) pValue).getWaypoint());
		} else {
			Util.typeError("Pos or TrackStep or NaviWaypoint", pValue);
		}
		return result;
	}

	// ==================================================================

	private Value stepNo(Value pValue) {
		return Quantity.valueOf(trackStep(pValue).getTrackStep().getStepNumber());
	}

	// ==================================================================

	private Value stepSourceStepNo(Value pValue) {
		return Quantity.valueOf(trackStep(pValue).getTrackStep().getSourceStepIndex());
	}
	
	// ==================================================================

	private Value angle(Value pValue) {
		return Quantity.valueOf(trackStep(pValue).getTrackStep().getAngleChange());
	}

	// ==================================================================

	private Value length(Value pValue) {
		return Quantity.valueOf(trackStep(pValue).getTrackStep().getStepLength());
	}

	// ==================================================================

	private Value dist(Value pValue) {
		return Quantity.valueOf(trackStep(pValue).getTrackStep().getDistance());
	}

	// ==================================================================

	private Value elev(Value pValue) {
		return Quantity.valueOf((trackStep(pValue).getTrackStep().getSElev() * 1.) / 4);
	}

	// ==================================================================

	private Value alts(Value pValue) {
		Value result = EMPTYLIST;
		TrackStepValue trackStepValue = trackStep(pValue);
		if (trackStepValue.getTrackStep().hasStepAlternatives()) {
			List<TrackStepI> alternariveSteps = trackStepValue.getTrackStep().getStepAlternatives();
			TrackStepValue array[] = new TrackStepValue[alternariveSteps.size()];
			int i = 0;
			for (TrackStepI step : alternariveSteps) {
				array[i++] = new TrackStepValue(step);
			}
			result = Util.valArrayToList(array);
		}
		return result;
	}

	// ==================================================================
	// == Attribute Container ===========================================
	// ==================================================================

	
	private SchemeBoolean hasAttribute(Value pValue1, Value pValue2) {
		SchemeBoolean result;
		NodeAttributeContainerValue nodeAttributeContainer = nodeAttributeContainer(pValue1);
		SchemeString attrib = Util.str(pValue2);
		result = nodeAttributeContainer.hasAttribute(attrib);
		return result;
	}
	
	// ==================================================================

	private SchemeString getAttribute(Value pValue1, Value pValue2) {
		SchemeString result;
		NodeAttributeContainerValue nodeAttributeContainer = nodeAttributeContainer(pValue1);
		SchemeString attrib = Util.str(pValue2);
		result = nodeAttributeContainer.getAttribute(attrib);
		return result;
	}

	// ==================================================================

	private Pair getAllAttribsFromContainer(Value pValue) {
		Pair result = Util.EMPTYLIST;
		NodeAttributeContainerValue nodeAttributeContainer = nodeAttributeContainer(pValue);
		SchemeString[] list = nodeAttributeContainer.getAttributes();
		if(list != null) {
			result = Util.valArrayToList(list);
		}
		return result;
	}
	
	// ==================================================================

	
	
}
