package rgnavigator.scripting;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import rgnavigator.core.exceptions.WaypointScriptException;
import rgnavigator.core.interfaces.NavigationContextI;
import sisc.data.Procedure;
import sisc.data.Symbol;
import sisc.data.Value;
import sisc.interpreter.AppContext;
import sisc.interpreter.Context;
import sisc.interpreter.Interpreter;
import sisc.interpreter.SchemeCaller;
import sisc.interpreter.SchemeException;
import sisc.util.Util;

public class NaviWaypointCalculator {
	private AppContext appContext;

//	private static TrackStep createStep(int lon, int lat, double angle) {
//		TrackStep step = new TrackStep(new Pos(lon, lat, (short) 1), null);
//		step.setAngleChange(angle);
//		return step;
//	}
//	
//	public static void main(String[] args) {
//
//		NaviWaypointCalculator testMe = new NaviWaypointCalculator();
//		NavigationContext navigationContext = new NavigationContext();
//		TrackStep step;
//		TrackStep altStep;
//		navigationContext.addStep(createStep(1,1,10));
//		navigationContext.addStep(createStep(3,5,0));
//
//		step = createStep(5,4,10);
//		altStep = createStep(5,5,10);
//		step.addAlternative(altStep);
//		altStep = createStep(3,8,10);
//		step.addAlternative(altStep);
//		navigationContext.addStep(step);
//
//		navigationContext.addStep(createStep(7,6,-30));
//
//		step = createStep(7,5,10);
//		altStep = createStep(5,6,10);
//		step.addAlternative(altStep);
//		altStep = createStep(8,7,10);
//		step.addAlternative(altStep);
//		navigationContext.addStep(step);
//
//		navigationContext.addStep(createStep(7,6,70));
//
//		testMe.calculateNaviWaypoints(navigationContext);
//	}

	// ====================================================
	public void calculateNaviWaypoints(NavigationContextI pNavigationContext, File pScriptFile) {
		appContext = new AppContext();
		try {
			InputStream codeStream = new FileInputStream(pScriptFile);
			InputStreamReader in = new InputStreamReader(codeStream, "UTF-8");
			final char[] buffer = new char[2000];
			StringBuffer code = new StringBuffer();
			for (;;) {
				int rsz = in.read(buffer, 0, buffer.length);
				if (rsz < 0)
					break;
				code.append(buffer, 0, rsz);
			}
			codeStream.close();
			appContext.addDefaultHeap();

			Context.execute(appContext, new SchemeCaller() {
				@Override
				public Object execute(Interpreter pInterpreter) throws SchemeException {
					Value value = null;
					try {
						NaviProcedures.Index index = new NaviProcedures.Index();
						index.bindAll(pInterpreter, appContext.toplevel_env);
						value = pInterpreter.eval(code.toString());
						Procedure proc = (Procedure) pInterpreter.lookup(Symbol.get("calc-navi-waypoints"), Util.TOPLEVEL);
						Value[] values = new Value[] { new NavigationContextValue(pNavigationContext) };
						value = pInterpreter.eval(proc, values);
					} catch (IOException e) {
						throw new WaypointScriptException(e);
					}
					return value;
				}
			});

		} catch (IOException | SchemeException e) {
			throw new WaypointScriptException(e);
		}
	}
}
