package rgnavigator.scripting;

import java.io.IOException;

import rgnavigator.core.interfaces.NodeAttributeContainerI;
import sisc.data.SchemeBoolean;
import sisc.data.SchemeString;
import sisc.data.Value;
import sisc.io.ValueWriter;
import sisc.modules.s2j.Util;

public class NodeAttributeContainerValue extends Value {

	private NodeAttributeContainerI nodeAttributeContainer;

	// =======================================================

	public NodeAttributeContainerValue(NodeAttributeContainerI pNodeAttributeContainer) {
		nodeAttributeContainer = pNodeAttributeContainer;
	}

	public SchemeString[] getAttributes() {
		SchemeString[] result = null;
		if (nodeAttributeContainer != null) {
			result = new SchemeString[nodeAttributeContainer.getAttributes().size()];
			int i = 0;
			for (String attrib : nodeAttributeContainer.getAttributes()) {
				result[i++] = new SchemeString(attrib);
			}
		}
		return result;

	}

	// =======================================================

	public SchemeString getAttribute(SchemeString pKey) {
		SchemeString result = null;
		if (nodeAttributeContainer != null) {
			String key = pKey.asString();
			if (nodeAttributeContainer.hasAttribute(key)) {
				result = new SchemeString(nodeAttributeContainer.getAttribute(key));
			}
		}
		return result;
	}

	// =======================================================

	public SchemeBoolean hasAttribute(SchemeString pKey) {
		SchemeBoolean result = Util.FALSE;
		if (nodeAttributeContainer != null && nodeAttributeContainer.hasAttribute(pKey.asString())) {
			result = Util.TRUE;
		}
		return result;
	}

	// =======================================================

	public void display(ValueWriter pValueWriter) throws IOException {
		pValueWriter.append("NodeAttributeContainerValue: ");
		if (nodeAttributeContainer != null) {
			for (String attrib : nodeAttributeContainer.getAttributes()) {
				pValueWriter.append(attrib).append("=>").append(nodeAttributeContainer.getAttribute(attrib)).append(", ");
			}
			pValueWriter.append("\n");
		}
	}

	// =======================================================
	@Override
	public boolean eqv(Object arg0) {
		boolean result = false;
		if (arg0 instanceof NodeAttributeContainerValue) {
			NodeAttributeContainerValue cmp = (NodeAttributeContainerValue) arg0;
			if (nodeAttributeContainer != null && cmp.nodeAttributeContainer != null) {
				result = nodeAttributeContainer.equals(cmp.nodeAttributeContainer);
			}
		}
		return result;
	}

	// =======================================================

}
