;; Calculate navigation points
;; Trekking-Profile
;; Version 0.1
;; This is a little SISC Scheme program
;; The full SISC interpreter is available, so you should be able to build quite complex functionality
;; The current version of this script is extremely simple, but it works so far for me.
;; I added some navigation related function within the Scheme interpreter and hope you will find all you need to build complex scripts.
;; I only develop Scheme code every 10 years or so, so my example might be very clumsy.
;; In this example the navigation text is in German. Modify the function direction-text-def to your liking.
;;
;; After parsing the code the java router code will call the function calc-navi-waypoints. Start you coding there.
;;-------------------------------------------- 
;; supported navigation related functions
;;
;;step? <var>
;; ====> #t if var instance of track-step

;;step-next <track-step>
;; ====> track-step

;;step-prev <track-step>
;; ====> track-step

;;step-pos <track-step> | <navi-waypoint> | <pos>
;; ====> pos

;;pos? <var>
;; ====> #t if var instance of pos

;;step-alts <track-step>
;; ====> list of track-step which are alternative steps at a junction

;;step-no <track-step>
;; ====> current step number within the list of navi track steps

;;step-source-step-no <track-step>
;; ====> current step number within the list of source track steps being analyzed for navigation. step-no and step-source-step-no might differ, 
;;       because some source track steps might be ignored during internal processing and will not result in a track step used for navigation.

;;step-angle <track-step>
;; ====> angle change in relation to the previous step (-180 <= angle <= 180)

;;step-length <track-step>
;; ====> length of this step in meters

;;step-distance <track-step>
;; ====> distance form the track start in meters

;;step-elev <track-step>
;; ====> elevation in meters

;;step-junction? <track-step>
;; ====> #t if step has alternatives and is therefore a junction

;;step-next-junction <track-step>
;; ====> next track-step being a junction

;;step-prev-junction <track-step>
;; ====> previous track-step being a junction

;;step-count <navigation-context>
;; ====> number of steps for the current track

;;step-attrib-container <track-step>
;; ====> attrib-container for this step. This will include all the OSM attributes for a position beein passed down from the router.

;;string-regex-replace-all <src> <regex> <replace>
;; ====> new string

;;string-regex-replace-first <src> <regex> <replace>
;; ====> new string

;;waypoint-new
;; ====> new waypoint. A way point contains the actual navigation information.

;;waypoint-value <waypoint> <attribute>
;; Supported attributes:
;; name, comment, desc,  symbol, type,  gpx_extension, ilat, ilon, selev
;; ====> value 

;;waypoint-value! <waypoint> <attribute> <value>

;;waypoint-add <navigation-context> <waypoint>
;; Add the way point to the global list of way points.

;;firststep <navigation-context>
;; ====> track-step

;;laststep <navigation-context>
;; ====> track-step

;;navicontext? <var>
;; ====> #t if var instance of navigation-context

;;hassteps <navigation-context>
;; ====> #t, if track has steps

;;attrib-container? <var>
;; ====> #t, if var is instance of attrib-container

;;attrib-container-all-attribs <attrib-container>
;; ====> list of attribute keys

;;attrib-container-attrib? <attrib-container> <attribute>
;; ====> #t, if container holds the given attribute

;;attrib-container-attrib <attrib-container> <attribute>
;; ====> string with the attribute value

;;=================================================
;;=================================================
;;=================================================
;;=================================================
;;=================================================

(begin
  (define direction-text-def
    (list
     (cons 'strait "Gradeaus")
     (cons 'half-left "Halb links")
     (cons 'left "Links")
     (cons 'sharp-left "Scharf links")
     (cons 'reverse "Umdrehen")
     (cons 'half-right "Halb rechts")
     (cons 'right "Rechts")
     (cons 'sharp-right "Scharf rechts")
     )
    )
  ;;=================================================
	   
  (define (direction-text dir lst)
    (if (null? lst)
	"unbekannt"
	(let* (
	       (element (car (car lst)))
	       (value (cdr (car lst)))
	       )
	  (if (eqv? dir element)
	      value
	      (direction-text dir (cdr lst))
	      )
	  )
	)
    )
  ;;=================================================
  (define strait-angle-value 25)
  
  ;;=================================================
  (define (calc-strait-angle-value next-step alt-steps)
    ;; Just a dummy so far
    (set! strait-angle-value 30)
    )

  ;;=================================================
  
  (define (dir-symbol angle)
    (cond 
     ((< (abs angle) strait-angle-value) 'strait)
     ((and (>= angle 20) (< angle 60)) 'half-left)
     ((and (>= angle 60) (< angle 120)) 'left)
     ((>= angle 120) 'sharp-left)
     ((> (abs angle) 179.9) 'reverse)
     ((and (<= angle (* -1 strait-angle-value)) (> angle -60)) 'half-right)
     ((and (<= angle -60) (> angle -120)) 'right)
     ((<= angle -120) 'sharp-right)
     )
    )
      
  ;;=================================================
  ;; Find a  step having a the biggest angle being smaller
  ;; then the orig step
  (define (step-angle-bigger origstep currentstep alts)
    (if (null? alts)
	(if (eqv? currentstep origstep)
	    '()
	    currentstep
	    )
	(let (
	      (checkstep (car alts))
	      )
	  (if (< (step-angle  checkstep) (step-angle origstep))
	      (if (>= (step-angle  checkstep) (step-angle currentstep))
		  (step-angle-smaller origstep checkstep (cdr alts))
		  (step-angle-smaller origstep currentstep (cdr alts))
		  )
	      (step-angle-smaller origstep currentstep (cdr alts))
	      )
	  )
	)
    )

  ;;=================================================
  ;; Find a  step having a the smalles angle being bigger
  ;; then the orig step
  (define (step-angle-smaller origstep currentstep alts)
    (if (null? alts)
	(if (eqv? currentstep origstep)
	    '()
	    currentstep
	    )
	(let (
	      (checkstep (car alts))
	      )
	  (if (> (step-angle  checkstep) (step-angle origstep))
	      (if (<= (step-angle  checkstep) (step-angle currentstep))
		  (step-angle-smaller origstep checkstep (cdr alts))
		  (step-angle-smaller origstep currentstep (cdr alts))
		  )
	      (step-angle-smaller origstep currentstep (cdr alts))
	      )
	  )
	)
    )

  ;;=================================================
  (define (process-step context step next-step)
    (let* (
	  (alts (step-alts step))
	  (smaller-step (step-angle-smaller next-step next-step alts))
	  (larger-step (step-angle-bigger next-step next-step alts))
	  (waypoint (waypoint-new (step-pos step)))
	  (attrib-cont (step-attrib-container step))
	  )
;;      (display attrib-cont)
;;      (if (not (null? (attrib-container-all-attribs attrib-cont)))
;;	  (display (attrib-container-attrib attrib-cont (car (attrib-container-all-attribs attrib-cont))))
;;	  )
	  
;;      (calc-strait-angle-value next-step  (cons (if (null? smaller-step) '() smaller-step) (if (null? larger-step) '() smaller-step)))
      (let (
	    (dirtext (direction-text (dir-symbol (step-angle next-step)) direction-text-def))
	    )
	(waypoint-value! waypoint 'name dirtext)
	(waypoint-value! waypoint 'reference (number->string (step-source-step-no step)))
	(waypoint-add context waypoint)
	)
      )
    )

  ;;=================================================

  (define (eval-steps context step)
    (if (step? step)
	(let (
	      (next-step (step-next step))
	      )
	  (let ()
	    (if (step-junction? step)
		(process-step context step next-step)
		)
	    (eval-steps context (step-next-junction step))
	    )
	  )
	)
    )
  
  ;;=================================================

  (define (calc-navi-waypoints context)
    (let* (
	   (step (firststep context))
	   )
;;      (stack-trace-on-error #t)
;;      (max-stack-trace-depth 1)
;;      (emit-debugging-symbols #t)
      
      (eval-steps context step)
      #t
      )
    )
  )
